# 🚀 programmering.no | 🤓 matematikk.as
# - Ser om et element eksisterer i en liste

# Verdier
liste = ["elefant", "ulv", "rev"]
dyr   = "elefant"

# Print liste
print(f"Liste: {liste}")

# Itererer gjennom listen
for el in liste:

    # El eksisterer i listen
    if el == dyr:

        # Print el
        print(f"Ja, {dyr} eksisterer i listen")
