# 🚀 programmering.no | 🤓 matematikk.as
# - Prøv å add et element til frozenset (går ikke)

# Variabler
sett = frozenset({"bil", "tog", "sykkel"})

# Print originalt sett
print(f"Originalt sett: {sett}")

# Prøv å add et element til frozenset (går ikke)
sett.add("fly")

# Print oppdatert sett
print(f"Oppdatert sett: {sett}")
