# 🚀 programmering.no | 🤓 matematikk.as
# - Ser om et element eksisterer i en liste

# Verdier
liste = ["elefant", "ulv", "rev"]
dyr   = "elefant"

# Ser om el eksisterer i listen
print(f"Liste: {liste}")
if dyr in liste:
    print(f"Ja, {dyr} eksisterer i listen")
else:
	print("Nei, elementet eksisterer ikke i listen")
